﻿/***********************************************************************************
* PROJECT: check_DiskQueueLength                                                   *
* AUTHOR: Alan Pipitone                                                            *
* DATE: 05/20/2012                                                                 *
* WEBSITE: http://www.alan-pipitone.com/                                           *
*                                                                                  *
* NB: This software has no warranty                                                *
************************************************************************************/

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Globalization;
using System.Diagnostics;

namespace check_DiskQueueLength
{
    class Program
    {
        static void Main(string[] args)
        {
            //initialize variables
            string unit = "";
            bool PhysicalDisk = false;
            float warning = 0;
            float critical = 0;
            float average = 0;
            float value = 0;
            int j = 0;
            bool arg_u_present = false;
            bool arg_w_present = false;
            bool arg_c_present = false;
            float[] values = new float[10];

            if (args.Length > 0)
            {
                //read the arguments
                for (int i = 0; i < args.Length; i++)
                {
                    switch (args[i])
                    {
                        case "-u":
                            if (args.Length > i + 1)
                            {
                                //save disk unit
                                unit = args[++i];
                                arg_u_present = true;
                            }
                            else
                            {
                                Console.WriteLine("ERROR in parsing argument -u: missing value");
                                Environment.Exit(3);
                            }
                            break;
                        case "-w":
                            if (args.Length > i + 1)
                            {
                                //save warning value
                                warning = Convert.ToSingle(args[++i], CultureInfo.InvariantCulture);
                                arg_w_present = true;
                            }
                            else
                            {
                                Console.WriteLine("ERROR in parsing argument -w: missing value");
                                Environment.Exit(3);
                            }
                            break;
                        case "-c":
                            if (args.Length > i + 1)
                            {
                                //save critical value
                                critical = Convert.ToSingle(args[++i], CultureInfo.InvariantCulture);
                                arg_c_present = true;
                            }
                            else
                            {
                                Console.WriteLine("ERROR in parsing argument -c: missing value");
                                Environment.Exit(3);
                            }
                            break;
                        case "--PhysicalDisk":
                            //set Category Name "PhysicalDisk" to true
                            PhysicalDisk = true;
                            break;
                        case "-h":
                            //print the help
                            Manual();
                            Environment.Exit(3);
                            break;
                        default:
                            //invalid argument
                            Console.WriteLine("ERROR: argument " + args[i] + " isn't valid, type check_DiskQueueLength.exe -h for the help");
                            Environment.Exit(3);
                            break;
                    }
                }

                //check if -u <disk unit> (mandatory argument) is present
                if(arg_u_present == false)
                {
                    Console.WriteLine("ERROR: -u <disk unit> is a mandatory argument, type check_DiskQueueLength.exe -h for the help");
                    Environment.Exit(3);
                }

                //check if -w <warning value> (mandatory argument) is present
                if (arg_w_present == false)
                {
                    Console.WriteLine("ERROR: -w <warning value> is a mandatory argument, type check_DiskQueueLength.exe -h for the help");
                    Environment.Exit(3);
                }

                //check if -c <critical value> (mandatory argument) is present
                if (arg_c_present == false)
                {
                    Console.WriteLine("ERROR: -c <critical value> is a mandatory argument, type check_DiskQueueLength.exe -h for the help");
                    Environment.Exit(3);
                }

                try
                {
                    //create a new PerformanceCounter object
                    PerformanceCounter diskQueue;

                    if (PhysicalDisk == true)
                    {
                        //initialize the PerformanceCounter object to get Avg. Disk Queue Length on PhysicalDisk
                        //diskQueue = new PerformanceCounter("PhysicalDisk", "Avg. Disk sec/Transfer", unit);
                        diskQueue = new PerformanceCounter("PhysicalDisk", "Avg. Disk Queue Length", unit);
                    }
                    else
                    {
                        //initialize the PerformanceCounter object to get Avg. Disk Queue Length on LogicalDisk
                        diskQueue = new PerformanceCounter("LogicalDisk", "Avg. Disk Queue Length", unit);
                    }

                    //It takes 15 seconds to finish (in this way we can get a good average)
                    for (j = 0; j < 15; j++)
                    {
                        //add current Avg. Disk Queue Length
                        value = value + diskQueue.NextValue();

                        //wait 1 sec (if you wait less your data will be unreliable)
                        Thread.Sleep(1000);

                    }

                    //calculate the average
                    average = value / j;

                    if (average >= critical) //print critical message with performance data
                    {
                        Console.WriteLine("CRITICAL: Avg. Disk Queue Length is " + String.Format("{0:0.0000}", average) + "|AvgDiskQueueLength=" + String.Format("{0:0.0000}", average) + ";" + warning + ";" + critical + ";;");
                        Environment.Exit(2);
                    }
                    else if (average >= warning) //print warning message with performance data
                    {
                        Console.WriteLine("WARNING: Avg. Disk Queue Length is " + String.Format("{0:0.0000}", average) + "|AvgDiskQueueLength=" + String.Format("{0:0.0000}", average) + ";" + warning + ";" + critical + ";;");
                        Environment.Exit(1);
                    }
                    else //print ok message with performance data
                    {
                        Console.WriteLine("OK: Avg. Disk Queue Length is " + String.Format("{0:0.0000}", average) + "|AvgDiskQueueLength=" + String.Format("{0:0.0000}", average) + ";" + warning + ";" + critical + ";;");
                        Environment.Exit(0);
                    }
                }
                catch (Exception ex)
                {
                    //print exception message with performance data
                    Console.WriteLine("UNKNOWN: " + ex.Message + "|AvgDiskQueueLength=0;" + warning + ";" + critical + ";;");
                    Environment.Exit(3);
                }
            }
            else
            {
                //print the help
                Manual();
            }
        }

        private static void Manual()
        {
            string help = "AUTHOR: Alan Pipitone\n\n";
            help = help + "WEBSITE: http://www.alan-pipitone.com/\n\n";
            help = help + "Arguments of check_DiskQueueLength:\n\n";
            help = help + "    -u \"<Disk Unit>\"\n";
            help = help + "    -w <warning value>\n";
            help = help + "    -c <critical value>\n";
            help = help + "    --PhysicalDisk to get Avg. of Physical Disc(s)\n";
            help = help + "\n";
            help = help + "If you don't add --PhysicalDisk then you will get Avg. Disk Queue Length of Logical Disk.\n";
            help = help + "\n";
            help = help + "Examples:\n";
            help = help + "\n";
            help = help + "check_DiskQueueLength.exe -u \"0 C:\" -w 2 -c 4 --PhysicalDisk\n\n";
            help = help + "check_DiskQueueLength.exe -u _Total -w 2 -c 4 --PhysicalDisk\n\n";
            help = help + "check_DiskQueueLength.exe -u C: -w 2 -c 4\n\n";
            help = help + "check_DiskQueueLength.exe -u _Total -w 2 -c 4\n\n";
            help = help + "NB: If you don't know what to put as <disk unit> after -u (especially if you use the switch --PhysicalDisk), please use the Performance Monitor of your Windows server to see the correct name of disk unit.";
            //-u "0 C:" -w  1 -c 2 --pd d
            

            Console.WriteLine(help);
        }
    }

}
